package edu.washington.cs.atomicdebug.examples;

/**
 * An example of a race condition using the classic bank account example. Two
 * threads try to deposit $200 to a bank account simultaneously. The race
 * condition may cause $200 to "disappear."
 * 
 * @author Laura Effinger-Dean
 */
public class BankAccountRaceCondition {
	public static void main(String[] args) {
		for (int i = 0; i < 10; i++) {
			// create a new bank account
			// change this to BankAccountSolved in order to use the "solution" code!
			final BankAccount b = new BankAccount(500);

			// two threads try to deposit $200 to the account simultaneously
			Thread t1 = new Thread() {
				public void run() {
					b.deposit(200);
				}
			};
			Thread t2 = new Thread() {
				public void run() {
					b.deposit(200);
				}
			};
			t1.start();
			t2.start();

			// wait for both threads to finish
			try {
				t1.join();
				t2.join();
			} catch (InterruptedException e) {
				e.printStackTrace();
			}

			// check the final account balance. Should be $900.
			System.out.println("Account balance (run #" + (i+1) + "): "
					+ b.balance());
		}
	}
}
