package edu.washington.cs.atomicdebug.examples;

import edu.washington.cs.atomicdebug.AtomicDebug;

/**
 * This program creates two threads, both of which have access to a variable i.
 * They then each try to increment i five times and print its new value each
 * time. Not the most practical program, but it demonstrates some of the odd
 * behavior race conditions may cause.
 * 
 * This "solution" has some thread sleeps and atomic blocks added to demonstrate
 * and fix race conditions.
 * 
 * @author Laura Effinger-Dean
 */
public class TwoThreadsSolved extends Thread {
	static int i = 0;

	public void run() {
		try {
			for (int j = 0; j < 5; j++) {
				AtomicDebug.enterAtomic();
				System.out.println(getName() + ": " + i);
				int temp = i;
				Thread.sleep(10);
				i = temp + 1;
				AtomicDebug.exitAtomic();
				Thread.sleep(10);
			}
		} catch (InterruptedException e) {
			// Ignore exception
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		TwoThreadsSolved t1 = new TwoThreadsSolved();
		TwoThreadsSolved t2 = new TwoThreadsSolved();
		t1.start();
		t2.start();
	}
}